"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const file_utils_1 = require("./file-utils");
const interface_definition_1 = require("./interface-definition");
const result_report_1 = require("./result-report");
const chalk_1 = __importDefault(require("chalk"));
const command_line_args_1 = __importDefault(require("command-line-args"));
const optionDefinitions = [
    { name: 'verbose', alias: 'v', type: Boolean },
    { name: 'tsconfig', alias: 't', type: String },
    { name: 'src', alias: 's', type: String },
    { name: 'json', alias: 'j', type: String },
];
const options = command_line_args_1.default(optionDefinitions);
if (!options.src || !options.tsconfig || !options.json) {
    throw new Error('ArgumentError');
}
file_utils_1.loadJson(options.json).then((data) => {
    if (typeof data !== 'object' && data === null) {
        throw new Error('Root of provided JSON structure is not an object.');
    }
    console.log(chalk_1.default.green("Loading project..."));
    const interfaceDefinition = new interface_definition_1.InterfaceDefinition({
        project: {
            tsConfigFilePath: options.tsconfig
        },
        sourceFilePath: options.src,
        targetData: data
    });
    try {
        console.log(chalk_1.default.green("Validating..."));
        const result = interfaceDefinition.compareToTarget();
        if (result.valid) {
            console.log(chalk_1.default.bold(`=== Validation Result ===`));
            console.log(chalk_1.default.green("Valid"));
        }
        else {
            if (result.asts) {
                const report = new result_report_1.ResultReport(result.asts).generateReport();
                console.error(chalk_1.default.bold(`=== Validation Detail ===`));
                report.forEach((r) => {
                    console.error("JSON Path  : ", chalk_1.default.yellowBright(r.objectPathIdentifier.join('.')));
                    console.error("TS FilePath: ", `${chalk_1.default.yellow(r.sourceFileName)}:${chalk_1.default.green(r.sourceFilePos.line.toString())}:${r.sourceFilePos.column.toString()}`);
                    if (options.verbose) {
                        console.error(chalk_1.default.dim(r.text));
                        console.error(chalk_1.default.dim(JSON.stringify(r.objectValue, null, 2)));
                    }
                });
                console.log(`\n${report.length} error(s) found`);
            }
            console.error(chalk_1.default.bold(`=== Validation Result ===`));
            console.error(chalk_1.default.redBright.bold("Not Valid"));
            process.exitCode = 1;
        }
    }
    catch (e) {
        console.error("Error:", chalk_1.default.redBright.bold(e.toString()));
    }
});
//# sourceMappingURL=index.js.map